@@ -1814,7 +1814,8 @@ class JsonManipulatorTest extends TestCase
 ', $manipulator->getContents());
     }
 
-    public function testAddConfigWithPackage() {
+    public function testAddConfigWithPackage()
+    {
         $manipulator = new JsonManipulator('{
     "repositories": [
         {
@@ -1935,7 +1936,7 @@ class JsonManipulatorTest extends TestCase
 ", $manipulator->getContents());
     }
 
-    public function testAddRepositoryCanAdd()
+    public function testAddRepositoryCanAppend()
     {
         $manipulator = new JsonManipulator('{
     "repositories": {
@@ -1946,7 +1947,7 @@ class JsonManipulatorTest extends TestCase
     }
 }');
 
-        $this->assertTrue($manipulator->addRepository('bar', array('type' => 'composer')));
+        $this->assertTrue($manipulator->addRepository('bar', array('type' => 'composer'), true));
         $this->assertEquals('{
     "repositories": {
         "foo": {
@@ -1961,6 +1962,32 @@ class JsonManipulatorTest extends TestCase
 ', $manipulator->getContents());
     }
 
+    public function testAddRepositoryCanPrepend()
+    {
+        $manipulator = new JsonManipulator('{
+    "repositories": {
+        "foo": {
+            "type": "vcs",
+            "url": "lala"
+        }
+    }
+}');
+
+        $this->assertTrue($manipulator->addRepository('bar', array('type' => 'composer'), false));
+        $this->assertEquals('{
+    "repositories": {
+        "bar": {
+            "type": "composer"
+        },
+        "foo": {
+            "type": "vcs",
+            "url": "lala"
+        }
+    }
+}
+', $manipulator->getContents());
+    }
+
     public function testAddRepositoryCanOverrideDeepRepos()
     {
         $manipulator = new JsonManipulator('{
@@ -2411,6 +2438,48 @@ class JsonManipulatorTest extends TestCase
 ', $manipulator->getContents());
     }
 
+    public function testRemoveMainKeyIfEmpty()
+    {
+        $manipulator = new JsonManipulator('{
+    "repositories": [
+    ],
+    "require": {
+        "package/a": "*",
+        "package/b": "*",
+        "package/c": "*"
+    },
+    "foo": "bar",
+    "require-dev": {
+    }
+}');
+
+        $this->assertTrue($manipulator->removeMainKeyIfEmpty('repositories'));
+        $this->assertEquals('{
+    "require": {
+        "package/a": "*",
+        "package/b": "*",
+        "package/c": "*"
+    },
+    "foo": "bar",
+    "require-dev": {
+    }
+}
+', $manipulator->getContents());
+
+        $this->assertTrue($manipulator->removeMainKeyIfEmpty('foo'));
+        $this->assertTrue($manipulator->removeMainKeyIfEmpty('require'));
+        $this->assertTrue($manipulator->removeMainKeyIfEmpty('require-dev'));
+        $this->assertEquals('{
+    "require": {
+        "package/a": "*",
+        "package/b": "*",
+        "package/c": "*"
+    },
+    "foo": "bar"
+}
+', $manipulator->getContents());
+    }
+
     public function testRemoveMainKeyRemovesKeyWhereValueIsNull()
     {
         $manipulator = new JsonManipulator(json_encode(array(
